import 'dart:convert';
import 'dart:io';
import 'package:path/path.dart' as p;
import 'package:get/get.dart';
import 'package:speed_safe/app/config/app_config.dart';
import 'package:speed_safe/app/util/system.dart';

class ConfigController extends GetxController {
  AppConfig appConfig = AppConfig.defaults();
  String configFileName = "config.json";
  late File configFile = File(p.join(configPath, configFileName));

  Future<void> init() async {
    if (SystemUtil.fileExists(configFileName)) {
      final jsonString = jsonEncode(appConfig.toJson());
      await writeConfig(jsonString);
    } else {
      final josnString = await configFile.readAsString();
      appConfig = AppConfig.fromJson(jsonDecode(josnString));
    }
  }

  //保存并更新配置文件
  Future<void> saveConfig(AppConfig config) async {
    appConfig = config;
    final jsonString = jsonEncode(appConfig.toJson());
    await writeConfig(jsonString);
  }

  Future<void> writeConfig(String jsonString) async {
    SystemUtil.deleteFileIfExists(
        configFile.path, 'Deleting config file: $configFileName');
    await configFile.writeAsString(jsonString);
  }
}
